@@ -1,8 +1,6 @@
 /* indices.h -- Functions defined in indices.c.
-   $Id: indices.h 7013 2016-02-13 21:19:19Z gavin $
 
-   Copyright 1993, 1997, 2004, 2007, 2013, 2014, 2015, 2016
-   Free Software Foundation, Inc.
+   Copyright 1993-2022 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -31,12 +29,14 @@ REFERENCE **apropos_in_all_indices (char *search_string, int inform);
 /* User visible functions declared in indices.c. */
 void info_index_search (WINDOW *window, int count);
 void info_index_apropos (WINDOW *window, int count);
-void next_index_match (FILE_BUFFER *fb, char *string, int offset, int dir,
-                  REFERENCE **result, int *found_offset, int *match_offset);
+REFERENCE *next_index_match (FILE_BUFFER *fb, char *string,
+                             int offset, int dir,
+                             int *found_offset, int *match_offset);
 void report_index_match (int i, int match_offset);
 REFERENCE *look_in_indices (FILE_BUFFER *fb, char *string, int sloppy);
+NODE *create_virtual_index (FILE_BUFFER *file_buffer, char *index_search);
 
 #define APROPOS_NONE \
-   N_("No available info files have '%s' in their indices.")
+   N_("No available info files have '%s' in their indices")
 
 #endif /* not INFO_INDICES_H */
