@@ -1,12 +1,11 @@
 /* signals.h -- header to include system dependent signal definitions.
-   $Id: signals.h,v 1.2 2004/03/20 01:44:23 karl Exp $
 
-   Copyright (C) 1993, 1994, 1995, 1997, 2002, 2004 Free Software Foundation, Inc.
+   Copyright 1993-2022 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,10 +13,9 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-   Originally written by Brian Fox (bfox@ai.mit.edu). */
+   Originally written by Brian Fox. */
 
 #ifndef INFO_SIGNALS_H
 #define INFO_SIGNALS_H
@@ -25,6 +23,9 @@
 #include <sys/types.h>
 #include <signal.h>
 
+void signal_block_winch (void);
+void signal_unblock_winch (void);
+
 /* For sysV68 --phdm@info.ucl.ac.be.  */
 #if !defined (SIGCHLD) && defined (SIGCLD)
 #define SIGCHLD SIGCLD
