@@ -1,7 +1,7 @@
 use vars qw(%result_texis %result_texts %result_trees %result_errors 
    %result_indices %result_sectioning %result_nodes %result_menus
    %result_floats %result_converted %result_converted_errors 
-   %result_elements %result_directions_text);
+   %result_elements %result_directions_text %result_indices_sort_strings);
 
 use utf8;
 
@@ -10,25 +10,24 @@ $result_trees{'two_lines'} = {
     {
       'contents' => [
         {
-          'parent' => {},
-          'text' => 'text l 1
+          'contents' => [
+            {
+              'text' => 'text l 1
 '
-        },
-        {
-          'parent' => {},
-          'text' => 'text l 2
+            },
+            {
+              'text' => 'text l 2
 '
+            }
+          ],
+          'type' => 'paragraph'
         }
       ],
-      'parent' => {},
-      'type' => 'paragraph'
+      'type' => 'before_node_section'
     }
   ],
-  'type' => 'text_root'
+  'type' => 'document_root'
 };
-$result_trees{'two_lines'}{'contents'}[0]{'contents'}[0]{'parent'} = $result_trees{'two_lines'}{'contents'}[0];
-$result_trees{'two_lines'}{'contents'}[0]{'contents'}[1]{'parent'} = $result_trees{'two_lines'}{'contents'}[0];
-$result_trees{'two_lines'}{'contents'}[0]{'parent'} = $result_trees{'two_lines'};
 
 $result_texis{'two_lines'} = 'text l 1
 text l 2
@@ -42,6 +41,9 @@ text l 2
 $result_errors{'two_lines'} = [];
 
 
+$result_floats{'two_lines'} = {};
+
+
 
 $result_converted{'plaintext'}->{'two_lines'} = 'text l 1 text l 2
 ';
